module config

import os
import utils

pub struct Config {
	pub:
	host string
	port int
}

pub fn get_conf() Config {
	return populate_config()
}

fn populate_config() Config {
	home := os.getenv('HOME')
	possible_paths := [
		'./server.pgons',
		'./config/server.pgons',
		'../server.pgons',
		'../config/server.pgons'
		'${home}/.server.pgons',
		'${home}/server.pgons',
		'${home}/.config/server.pgons'
	]
	for ppath in possible_paths {
		if os.exists(ppath) {
			return read_config_file(ppath)
		}
	}
	utils.dbg('could not find a config file in either possible places: ${possible_paths}')
	return Config{
		host: utils.genv_string('PGONS_HOST', '0.0.0.0')
		port: utils.genv_string('PGONS_PORT', '1978').int()
	}
}

fn read_config_file(file string) Config {
	utils.dbg('found config file: ${file}')
	lines := os.read_lines(file) or {
		panic(err)
	}

	/* FIXME: config fields allowed. should be solved in a nicer way i think */
	mut t_host := utils.genv_string('PGONS_HOST', '0.0.0.0')
	mut t_port := utils.genv_string('PGONS_PORT', '1978').int()

	utils.dbg('parsing ${lines} into our config structure')
	for line in lines {
		sp := line.split(':')
		match true {
			sp[0] == 'host' { t_host = sp[1] }
			sp[0] == 'port' { t_port = sp[1].int() }
			else { utils.dbg('${sp[0]} is not a known config key') }
		}
	}

	return Config{
		host: t_host
		port: t_port
	}
}
