module main

import io
import net
import utils
import config

const pgons_host = utils.genv_string('PGONS_HOST', '0.0.0.0')
const pgons_port = utils.genv_string('PGONS_PORT', '1978').int()

const pgons_config = config.get_conf()

fn main() {
	mut server := net.listen_tcp(.ip, '${pgons_host}:${pgons_port}') or {
		panic(err)
	}
	addr := server.addr() or {
		panic(err)
	}
	println('listening on ${addr}')
	for {
		mut socket := server.accept() or {
			panic('socket failed: ${err}')
		}
		spawn handle_client(mut socket)
	}
}

fn handle_client(mut socket net.TcpConn) {
	defer {
		socket.close() or { panic(err) }
	}

	client_addr := socket.peer_addr() or { return }
	eprintln('> new client: ${client_addr}')
	mut reader := io.new_buffered_reader(reader: socket)
	defer {
		unsafe {
			reader.free()
		}
	}

	socket.write_string('server: hello\n') or { return }
	for {
		received_line := reader.read_line() or { return }
		if received_line == '' {
			return
		}
		println('client ${client_addr}: ${received_line}')
		socket.write_string('server: ${received_line}\n') or { return }
	}
}
